////////////////////////////////////////////////////////////////////////////
// pkelmer.cpp
// this program is GPL
///////////////////////////////////////////////////////////////////////////

#include "pkelmer.h"
#include "spectrum.h"
#include <qstring.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <malloc.h>
#include <stdio.h>


SPFile::SPFile(QString AFile) :
        Spectrum(AFile)
{
FName = AFile;
needFill = false;
mklp = true;
Load(AFile);

}

void SPFile::Load(QString AFile)
{
 if((f = fopen(AFile.latin1(), "r"))==NULL)
    {
    QMessageBox::information(NULL, "Error", "Unable to open File");
    return;
    }
 spectraFile = new QTextIStream(f);
 fxpos = 0;
 
 LoadData();
}

void SPFile::LoadData()
{
nLinha = 0;
Data = false;

while(!feof(f) && Valid && (!Data))
{
  nLinha++;
  linhaAtual = spectraFile->readLine();
  fxpos = fxpos + linhaAtual.length() + 2;
  
  switch(GetType())
  {
   case tiTitle:
    {
    Title = linhaAtual.left(20);
    break;
    }
    case tiOrigin:
    {
    Origin = GetValue();
    break;
    }
    case tiOwner:
    {
    Owner = GetValue();
    break;
    }
    case tiLastX:
    {
    lastx = GetValue().toFloat();
    break;
    }
    case tiXunits:
    {
    xunits = getxunits();
    break;
    }
    case tiYunits:
    {
    yunits = getyunits();
    break;
    }
    case tiTime:
    {
    Time = GetValue();
    break;
    }
    case tiDate:
    {
    Date = GetValue();
    break;
    }
    case tiCreated:
    {
    Created = GetValue();
    break;
    }
    case tiNpoints:
    {
    Npoints = GetValue().toFloat();
    break;
    }
    case tiXfactor:
    {
    xFactor = GetValue().toFloat();
    break;
    }
    case tiYfactor:
    {
    yFactor = GetValue().toFloat();
    break;
    }
    case tiFirstX:
    {
    firstx = GetValue().toFloat();
    break;
    }
    case tiFirstY:
    {
    firsty = GetValue().toFloat();
    break;
    }
    case tiMaxY:
    {
    maxY = GetValue().toFloat();
    qWarning(GetValue());
    break;
    }
    case tiMinY:
    {
    minY = GetValue().toFloat();
    qWarning(GetValue());
    break;
    }
    case tiData:
    ProcessPoints();
  } 

}

}

YUnits SPFile::getyunits()
{
if(linhaAtual=="%T")
  return transmittance; else
  return absorbance;
}

XUnits SPFile::getxunits()
{
if(linhaAtual=="CM-1")
  return cmRecipro; else
  return other;
}

void SPFile::getY(long int py, float fx, float percent)
{
QString tmp;
int i;

if((maxY != 0) && (minY != 0))
  {
  tmp =  tmp.setNum(py*yFactor);
  tmp =  tmp.setNum(((tmp.toFloat()-minY)/percent));
  } else
  tmp = tmp.setNum((py*yFactor)+4);

  if(lp > 11)
   {
   tyValues.append(tmp);
   txValues.append(tmp.setNum(fx));
   lp = 0;
   }

  
}

void SPFile::ProcessPoints()
{
 long int fy;
 float fx, percent;
 int i;


 incFactor = (lastx-firsty)/(Npoints-1);
 percent = (maxY-minY)/100;
 Created = Date + " " + Time;

 txValues = QStringList();
 tyValues = QStringList();

 fx = lastx;
 lp = 12;
 fy = 0L;

 while(!feof(f))
   {
   fread(&fy, sizeof(fy), 1, f);
   getY(fy, fx, percent);

   fx += xFactor;
   if(mklp)
     lp++; else
     lp = 12;
     
   }


   i = (txValues.count()-1); 
   while(i >= 0)
    {
    xValues.append(txValues[i]);
    yValues.append(tyValues[i]);
    i--; 
    }

    waveNumbers = xValues;
    transmitancia = yValues;
   
   

 
}

QString& SPFile::GetValue()
{
return linhaAtual;
}

int SPFile::GetType()
{
QString tmp;
int Result;

  linhaAtual = linhaAtual.upper();
  if(linhaAtual.contains("'"))
   linhaAtual = linhaAtual.left(linhaAtual.find("'") - 1);

  Result = tiNone;
  if(Data) 
    Result = tiData;

  if(nLinha == 1)
     {
      tmp = linhaAtual.stripWhiteSpace();
      tmp = tmp.left(5);
      
      if(tmp != "PE IR") // this is not a pkelmer valid file
        Valid = false;
     }


  if(nLinha == 1)
     Result = tiTitle;
  if(nLinha == 34)
     Result = tiOrigin;
  if(nLinha == 8)
     Result = tiOwner;
  if(nLinha == 43)
     Result = tiXunits;
  if(nLinha == 5)
     Result = tiTime;
  if(nLinha == 6)
     Result = tiDate;
  if(nLinha == 44)
     Result = tiYunits;
  if(nLinha == 51)
     Result = tiMaxY;
  if(nLinha == 52)
     Result = tiMinY;
  if(nLinha == 49)
     Result = tiNpoints;
  if(nLinha == 48)
     Result = tiXfactor;
  if(nLinha == 45)
     Result = tiYfactor;
  if(nLinha == 10)
     Result = tiFirstX;
  if(nLinha == 47)
     Result = tiLastX;
  if(linhaAtual.contains("#DATA"))
   {
     Result = tiData;
     Data = true;
   }
return Result;
}

SPFile::~SPFile()
{
 if(spectraFile)
   delete spectraFile;
}
